"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.validModel = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _types = require("./types");

Object.keys(_types).forEach(function (key) {
    if (key === "default" || key === "__esModule") return;
    Object.defineProperty(exports, key, {
        enumerable: true,
        get: function get() {
            return _types[key];
        }
    });
});

var _util = require("./util");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var createNewReferenceAccumulator = function createNewReferenceAccumulator(currentReferenceAccumulator, typeSchema) {
    if (!typeSchema["name"]) {
        return currentReferenceAccumulator;
    }
    var newReferenceAccumulator = Object.assign({}, currentReferenceAccumulator, _defineProperty({}, typeSchema["name"], typeSchema));
    return newReferenceAccumulator;
};
var mergeSchemas = function mergeSchemas(referencingSchema, referenceSchema) {
    var overWriteableProperties = ["nullAllowed", "undefinedAllowed", "typeFailureError", "restriction", "withContext"];
    var newSchema = Object.assign({}, referencingSchema);
    overWriteableProperties.map(function (propertyName) {
        if (referenceSchema[propertyName] !== undefined) {
            newSchema[propertyName] = referenceSchema[propertyName];
        }
    });
    return newSchema;
};
var getTypeOfSchema = function getTypeOfSchema(typeSchema) {
    var isObject = !(0, _util.isUndefined)(typeSchema.objectProperties);
    var isArray = !(0, _util.isUndefined)(typeSchema.arrayElementType);
    var isPrimitive = !(0, _util.isUndefined)(typeSchema.primitiveType);
    var isUnion = !(0, _util.isUndefined)(typeSchema.unionTypes);
    var isReference = !(0, _util.isUndefined)(typeSchema.referenceName);
    var isMap = !(0, _util.isUndefined)(typeSchema.mapValueType);
    var isAny = typeSchema.isAny;
    var kindOfTypeSchema = isObject && !isArray && !isPrimitive && !isUnion && !isReference && !isMap && !isAny ? _types.kindOfSchema.object : !isObject && isArray && !isPrimitive && !isUnion && !isReference && !isMap && !isAny ? _types.kindOfSchema.array : !isObject && !isArray && isPrimitive && !isUnion && !isReference && !isMap && !isAny ? _types.kindOfSchema.primitive : !isObject && !isArray && !isPrimitive && isUnion && !isReference && !isMap && !isAny ? _types.kindOfSchema.union : !isObject && !isArray && !isPrimitive && !isUnion && isReference && !isMap && !isAny ? _types.kindOfSchema.reference : !isObject && !isArray && !isPrimitive && !isUnion && !isReference && isMap && !isAny ? _types.kindOfSchema.map : !isObject && !isArray && !isPrimitive && !isUnion && !isReference && !isMap && isAny ? _types.kindOfSchema.any : null;
    return kindOfTypeSchema;
};
var validModelInternal = function validModelInternal(typeSchema, references) {
    return function (modelInstance) {
        var modelInstanceFollowsRestriction = function modelInstanceFollowsRestriction(restriction) {
            if (!restriction) {
                return Promise.resolve();
            }
            return Promise.resolve().then(function () {
                return restriction(modelInstance);
            });
        };
        return new Promise(function (resolve, reject) {
            var resolveIfRestrictionMet = function resolveIfRestrictionMet(restriction) {
                return modelInstanceFollowsRestriction(restriction).then(function () {
                    return resolve();
                }).catch(function (error) {
                    return reject(error);
                });
            };
            var kindOfTypeSchema = getTypeOfSchema(typeSchema);
            if ((0, _util.isNull)(kindOfTypeSchema)) {
                return reject(_types.schemaTypeError.invalidSchema);
            }
            ;
            {
                if (kindOfTypeSchema !== _types.kindOfSchema.reference) {
                    if ((0, _util.isNull)(modelInstance)) {
                        if (typeSchema.nullAllowed) {
                            return resolve();
                        }
                        return reject(typeSchema.typeFailureError || _types.schemaTypeError.nullField);
                    }
                    if ((0, _util.isUndefined)(modelInstance)) {
                        if (typeSchema.undefinedAllowed) {
                            return resolve();
                        }
                        return reject(typeSchema.typeFailureError || _types.schemaTypeError.undefinedField);
                    }
                }
            }
            if (typeSchema.withContext) {
                references = Object.assign({}, references, typeSchema.withContext());
            }
            switch (kindOfTypeSchema) {
                case _types.kindOfSchema.primitive:
                    {
                        var primitiveStructure = typeSchema;
                        var primitiveTypeStringName = _types.kindOfPrimitive[primitiveStructure.primitiveType];
                        if ((typeof modelInstance === "undefined" ? "undefined" : _typeof(modelInstance)) === primitiveTypeStringName) {
                            return resolveIfRestrictionMet(primitiveStructure.restriction);
                        }
                        return reject(primitiveStructure.typeFailureError || _types.schemaTypeError.primitiveFieldInvalid);
                    }
                case _types.kindOfSchema.array:
                    {
                        var _ret = function () {
                            var arrayStructure = typeSchema;
                            if (!Array.isArray(modelInstance)) {
                                return {
                                    v: reject(arrayStructure.typeFailureError || _types.schemaTypeError.arrayFieldInvalid)
                                };
                            } else {
                                var _ret2 = function () {
                                    var newReferences = createNewReferenceAccumulator(references, arrayStructure);
                                    var validArrayElement = validModelInternal(arrayStructure.arrayElementType, newReferences);
                                    return {
                                        v: {
                                            v: Promise.all(modelInstance.map(function (arrayElement) {
                                                return validArrayElement(arrayElement);
                                            })).then(function () {
                                                return resolveIfRestrictionMet(arrayStructure.restriction);
                                            }).catch(function (error) {
                                                return reject(error);
                                            })
                                        }
                                    };
                                }();

                                if ((typeof _ret2 === "undefined" ? "undefined" : _typeof(_ret2)) === "object") return _ret2.v;
                            }
                        }();

                        if ((typeof _ret === "undefined" ? "undefined" : _typeof(_ret)) === "object") return _ret.v;
                    }
                case _types.kindOfSchema.object:
                    {
                        var _ret3 = function () {
                            var objectStructure = typeSchema;
                            if ((typeof modelInstance === "undefined" ? "undefined" : _typeof(modelInstance)) !== "object") {
                                return {
                                    v: reject(objectStructure.typeFailureError || _types.schemaTypeError.objectFieldInvalid)
                                };
                            }
                            if (Array.isArray(modelInstance)) {
                                return {
                                    v: reject(objectStructure.typeFailureError || _types.schemaTypeError.objectFieldInvalid)
                                };
                            }
                            for (var modelProperty in modelInstance) {
                                if (!objectStructure.objectProperties[modelProperty]) {
                                    return {
                                        v: reject(objectStructure.typeFailureError || _types.schemaTypeError.objectHasExtraFields)
                                    };
                                }
                            }
                            var newReferences = createNewReferenceAccumulator(references, objectStructure);
                            return {
                                v: Promise.all(Object.keys(objectStructure.objectProperties).map(function (key) {
                                    var validProperty = validModelInternal(objectStructure.objectProperties[key], newReferences);
                                    return validProperty(modelInstance[key]);
                                })).then(function () {
                                    return resolveIfRestrictionMet(objectStructure.restriction);
                                }).catch(function (error) {
                                    return reject(error);
                                })
                            };
                        }();

                        if ((typeof _ret3 === "undefined" ? "undefined" : _typeof(_ret3)) === "object") return _ret3.v;
                    }
                case _types.kindOfSchema.map:
                    {
                        var _ret4 = function () {
                            var mapStructure = typeSchema;
                            if ((typeof modelInstance === "undefined" ? "undefined" : _typeof(modelInstance)) !== "object") {
                                return {
                                    v: reject(mapStructure.typeFailureError || _types.schemaTypeError.mapFieldInvalid)
                                };
                            }
                            if (Array.isArray(modelInstance)) {
                                return {
                                    v: reject(mapStructure.typeFailureError || _types.schemaTypeError.mapFieldInvalid)
                                };
                            }
                            var newReferences = createNewReferenceAccumulator(references, mapStructure);
                            var validProperty = validModelInternal(mapStructure.mapValueType, newReferences);
                            return {
                                v: Promise.all(Object.keys(modelInstance).map(function (key) {
                                    return validProperty(modelInstance[key]);
                                })).then(function () {
                                    return resolveIfRestrictionMet(mapStructure.restriction);
                                }).catch(function (error) {
                                    return reject(error);
                                })
                            };
                        }();

                        if ((typeof _ret4 === "undefined" ? "undefined" : _typeof(_ret4)) === "object") return _ret4.v;
                    }
                case _types.kindOfSchema.union:
                    {
                        var _ret5 = function () {
                            var unionStructure = typeSchema;
                            return {
                                v: (0, _util.anyPromise)(unionStructure.unionTypes.map(function (singleTypeFromUnion) {
                                    var validUnionType = validModelInternal(singleTypeFromUnion, references);
                                    return validUnionType(modelInstance);
                                })).then(function () {
                                    return resolve();
                                }).catch(function (arrayOfRejectedPromisesErrors) {
                                    return reject(unionStructure.typeFailureError || _types.schemaTypeError.unionHasNoMatchingType);
                                })
                            };
                        }();

                        if ((typeof _ret5 === "undefined" ? "undefined" : _typeof(_ret5)) === "object") return _ret5.v;
                    }
                case _types.kindOfSchema.reference:
                    {
                        var referenceStructure = typeSchema;
                        var referencingStructure = references[referenceStructure.referenceName];
                        if (!referencingStructure) {
                            return reject(_types.schemaTypeError.referenceNotFound);
                        }
                        var referenceActualTypeSchema = mergeSchemas(referencingStructure, referenceStructure);
                        return resolve(validModelInternal(referenceActualTypeSchema, references)(modelInstance));
                    }
                case _types.kindOfSchema.any:
                    {
                        var anyStructure = typeSchema;
                        return resolveIfRestrictionMet(anyStructure.restriction);
                    }
            }
        });
    };
};
var validModel = exports.validModel = function validModel(typeSchema) {
    return validModelInternal(typeSchema, {});
};
//# sourceMappingURL=main.js.map